/***************************************************/
/* Game Excelleration Kit vX                       */
/* Copyright (C) 2001 by HourGlass Software NonInc */
/***************************************************/
#ifndef __GEKVECTOR2__
#define __GEKVECTOR2__
/*! 
    \author MoJo
    \file GEKVector2.h
    \brief 2D Vector Class
*/

//! 2D Vector Class
struct GEKVECTOR2
{
public:
	union
	{
		float xy[2];
		float uv[2];
		struct { float x, y; };
		struct { float u, v; };
	};

public:
	inline GEKVECTOR2(void);
	inline GEKVECTOR2(const GEKVECTOR2 &kVector);
	inline GEKVECTOR2(const GEKVECTOR3 &kVector);
	inline GEKVECTOR2(const GEKVECTOR4 &kVector);
	inline GEKVECTOR2(float fX, float fY);

	inline float &operator [] (int iAxis);
	inline bool operator < (const GEKVECTOR2 &kVector) const;
	inline bool operator > (const GEKVECTOR2 &kVector) const;

	//@{
	//! Casting operator
	inline operator float * (void);
	inline operator const float * (void) const;
	//@}

	//@{
	//! Assignment operators
	inline GEKVECTOR2 operator = (const GEKVECTOR2 &kVector);
	inline GEKVECTOR2 operator = (const GEKVECTOR3 &kVector);
	inline GEKVECTOR2 operator = (const GEKVECTOR4 &kVector);
	inline operator -= (const GEKVECTOR2 &kVector);
	inline operator += (const GEKVECTOR2 &kVector);
	inline operator /= (float fScalar);
	inline operator *= (float fScalar);
	//@}

	//@{
	//! Binary Operators
	inline GEKVECTOR2 operator - (const GEKVECTOR2 &kVector) const;
	inline GEKVECTOR2 operator + (const GEKVECTOR2 &kVector) const;
	inline GEKVECTOR2 operator - (const GEKVECTOR2 &kVector);
	inline GEKVECTOR2 operator + (const GEKVECTOR2 &kVector);
	inline GEKVECTOR2 operator / (float fScalar) const;
	inline GEKVECTOR2 operator * (float fScalar) const;
	inline GEKVECTOR2 operator / (float fScalar);
	inline GEKVECTOR2 operator * (float fScalar);
	//@}

	inline void SetLength(float fLength);					//!< Sets the length of a vector
	inline float GetLengthSqr(void) const;					//!< Calculates the square length of a vector
	inline float GetLengthSqr(void);						//!< Calculates the square length of a vector
	inline float GetLength(void) const;						//!< Calculates the length of a vector
	inline float GetLength(void);							//!< Calculates the length of a vector

	inline float GetAngle(const GEKVECTOR2 &kVector);		//!< Calculates the angle between two vectors
	inline float Distance(const GEKVECTOR2 &kVector) const;	//!< Calculates the distance to a vector
	inline float Distance(const GEKVECTOR2 &kVector);		//!< Calculates the distance to a vector
	inline float Dot(const GEKVECTOR2 &kVector) const;		//!< Calculates the dot product of two vectors
	inline float Dot(const GEKVECTOR2 &kVector);			//!< Calculates the dot product of two vectors

	inline GEKVECTOR2 Lerp(const GEKVECTOR2 &kVector, float fFactor) const;	//!< Calculates the linear interpolation of two vectors
	inline GEKVECTOR2 Lerp(const GEKVECTOR2 &kVector, float fFactor);		//!< Calculates the linear interpolation of two vectors
	inline GEKVECTOR2 Normal(void) const;									//!< Calculates the normal of a vector
	inline GEKVECTOR2 Normal(void);											//!< Calculates the normal of a vector
	inline void Normalize(void);											//!< Normalizes a vector
};

inline GEKVECTOR2 operator - (const GEKVECTOR2 &kVector);	//!< Negative Operator
inline GEKVECTOR2 operator - (GEKVECTOR2 &kVector);			//!< Negative Operator

#endif