inline GEKVECTOR4 operator - (const GEKVECTOR4 &kVector)
{
	return GEKVECTOR4(-kVector.x,
					  -kVector.y,
					  -kVector.z,
					  -kVector.w);
}

inline GEKVECTOR4 operator - (GEKVECTOR4 &kVector)
{
	return GEKVECTOR4(-kVector.x,
					  -kVector.y,
					  -kVector.z,
					  -kVector.w);
}

inline GEKVECTOR4::GEKVECTOR4(void)
{
	x = y = z = w = 0.0f;
}

inline GEKVECTOR4::GEKVECTOR4(const GEKVECTOR2 &kVector)
{
	x = kVector.x;
	y = kVector.y;
	z = 0.0f;
	w = 1.0f;
}

inline GEKVECTOR4::GEKVECTOR4(const GEKVECTOR3 &kVector)
{
	x = kVector.x;
	y = kVector.y;
	z = kVector.z;
	w = 1.0f;
}

inline GEKVECTOR4::GEKVECTOR4(const GEKVECTOR4 &kVector)
{
	x = kVector.x;
	y = kVector.y;
	z = kVector.z;
	w = kVector.w;
}

inline GEKVECTOR4::GEKVECTOR4(float fX, float fY, float fZ, float fW)
{
	x = fX;
	y = fY;
	z = fZ;
	w = fW;
}

inline float &GEKVECTOR4::operator [] (int iAxis)
{
	return xyzw[iAxis];
}

inline bool GEKVECTOR4::operator < (const GEKVECTOR4 &kVector) const
{
	if((x < kVector.x)&&
	   (y < kVector.y)&&
	   (z < kVector.z)&&
	   (w < kVector.w)) return true;
	return false;
}

inline bool GEKVECTOR4::operator > (const GEKVECTOR4 &kVector) const
{
	if((x > kVector.x)&&
	   (y > kVector.y)&&
	   (z > kVector.z)&&
	   (w > kVector.w)) return true;
	return false;
}

inline GEKVECTOR4::operator float * (void)
{
	return &x;
}

inline GEKVECTOR4::operator const float * (void) const
{
	return &x;
}

inline GEKVECTOR4::operator -= (const GEKVECTOR4 &kVector)
{
	x -= kVector.x;
	y -= kVector.y;
	z -= kVector.z;
	w -= kVector.w;
}

inline GEKVECTOR4::operator += (const GEKVECTOR4 &kVector)
{
	x += kVector.x;
	y += kVector.y;
	z += kVector.z;
	w += kVector.w;
}

inline GEKVECTOR4::operator /= (float fScalar)
{
	x /= fScalar;
	y /= fScalar;
	z /= fScalar;
	w /= fScalar;
}

inline GEKVECTOR4::operator *= (float fScalar)
{
	x *= fScalar;
	y *= fScalar;
	z *= fScalar;
	w *= fScalar;
}

inline GEKVECTOR4 GEKVECTOR4::operator = (const GEKVECTOR2 &kVector)
{
	x = kVector.x;
	y = kVector.y;
	z = 0.0f;
	w = 1.0f;
	return *this;
}

inline GEKVECTOR4 GEKVECTOR4::operator = (const GEKVECTOR3 &kVector)
{
	x = kVector.x;
	y = kVector.y;
	z = kVector.z;
	w = 1.0f;
	return *this;
}

inline GEKVECTOR4 GEKVECTOR4::operator = (const GEKVECTOR4 &kVector)
{
	x = kVector.x;
	y = kVector.y;
	z = kVector.z;
	w = kVector.w;
	return *this;
}

inline GEKVECTOR4 GEKVECTOR4::operator - (const GEKVECTOR4 &kVector) const
{
	return GEKVECTOR4((x - kVector.x),
					  (y - kVector.y),
					  (z - kVector.z),
					  (w - kVector.w));
}

inline GEKVECTOR4 GEKVECTOR4::operator + (const GEKVECTOR4 &kVector) const
{
	return GEKVECTOR4((x + kVector.x),
					  (y + kVector.y),
					  (z + kVector.z),
					  (w + kVector.w));
}

inline GEKVECTOR4 GEKVECTOR4::operator - (const GEKVECTOR4 &kVector)
{
	return GEKVECTOR4((x - kVector.x),
					  (y - kVector.y),
					  (z - kVector.z),
					  (w - kVector.w));
}

inline GEKVECTOR4 GEKVECTOR4::operator + (const GEKVECTOR4 &kVector)
{
	return GEKVECTOR4((x + kVector.x),
					  (y + kVector.y),
					  (z + kVector.z),
					  (w + kVector.w));
}

inline GEKVECTOR4 GEKVECTOR4::operator / (float fScalar) const
{
	return GEKVECTOR4((x / fScalar),
				  	  (y / fScalar),
					  (z / fScalar),
					  (w / fScalar));
}

inline GEKVECTOR4 GEKVECTOR4::operator * (float fScalar) const
{
	return GEKVECTOR4((x * fScalar),
					  (y * fScalar),
					  (z * fScalar),
					  (w * fScalar));
}

inline GEKVECTOR4 GEKVECTOR4::operator / (float fScalar)
{
	return GEKVECTOR4((x / fScalar),
				  	  (y / fScalar),
					  (z / fScalar),
					  (w / fScalar));
}

inline GEKVECTOR4 GEKVECTOR4::operator * (float fScalar)
{
	return GEKVECTOR4((x * fScalar),
					  (y * fScalar),
					  (z * fScalar),
					  (w * fScalar));
}

inline float GEKVECTOR4::Dot(const GEKVECTOR4 &kVector) const
{
	return ((x * kVector.x) +
			(y * kVector.y) + 
			(z * kVector.z) + 
			(w * kVector.w));
}

inline float GEKVECTOR4::Dot(const GEKVECTOR4 &kVector)
{
	return ((x * kVector.x) +
			(y * kVector.y) + 
			(z * kVector.z) + 
			(w * kVector.w));
}

inline float GEKVECTOR4::GetLengthSqr(void) const
{
	return Dot(*this);
}

inline float GEKVECTOR4::GetLengthSqr(void)
{
	return Dot(*this);
}

inline float GEKVECTOR4::GetLength(void) const
{
	return SQRT(GetLengthSqr());
}

inline float GEKVECTOR4::GetLength(void)
{
	return SQRT(GetLengthSqr());
}

inline void GEKVECTOR4::SetLength(float fLength)
{
	*this *= (fLength / GetLength());
}

inline float GEKVECTOR4::Distance(const GEKVECTOR4 &kVector) const
{
	return (kVector - *this).GetLength();
}

inline float GEKVECTOR4::Distance(const GEKVECTOR4 &kVector)
{
	return (kVector - *this).GetLength();
}

inline float GEKVECTOR4::GetAngle(const GEKVECTOR4 &kVector)
{
	float fDot = Dot(kVector);
	float fMagnitude = (GetLength() * kVector.GetLength());
	return acosf(fDot / fMagnitude);
}

inline GEKVECTOR4 GEKVECTOR4::Cross(const GEKVECTOR4 &kVector) const
{
//	return GEKVECTOR2(((y * kVector.z) - (z * kVector.y)),
//					  ((z * kVector.x) - (x * kVector.z)),
//					  ((x * kVector.y) - (y * kVector.x)));
	return *this;
}

inline GEKVECTOR4 GEKVECTOR4::Cross(const GEKVECTOR4 &kVector)
{
//	return GEKVECTOR2(((y * kVector.z) - (z * kVector.y)),
//					  ((z * kVector.x) - (x * kVector.z)),
//					  ((x * kVector.y) - (y * kVector.x)));
	return *this;
}

inline GEKVECTOR4 GEKVECTOR4::Lerp(const GEKVECTOR4 &kVector, float fFactor) const
{
	return (*this + ((kVector - *this) * fFactor));
}

inline GEKVECTOR4 GEKVECTOR4::Lerp(const GEKVECTOR4 &kVector, float fFactor)
{
	return (*this + ((kVector - *this) * fFactor));
}

inline GEKVECTOR4 GEKVECTOR4::Normal(void) const
{
	float fLength = GetLength();
	return (*this * (1.0f / fLength));
}

inline GEKVECTOR4 GEKVECTOR4::Normal(void)
{
	float fLength = GetLength();
	return (*this * (1.0f / fLength));
}

inline void GEKVECTOR4::Normalize(void)
{
	*this = Normal();
}