#include "..\GEKXMath.h"

GEKMATRIX GEKMATRIX::operator * (const GEKMATRIX &kMatrix)
{
	return GEKMATRIX(((_11 * kMatrix._11) + (_12 * kMatrix._21) + (_13 * kMatrix._31) + (_14 * kMatrix._41)),
					 ((_11 * kMatrix._12) + (_12 * kMatrix._22) + (_13 * kMatrix._32) + (_14 * kMatrix._42)),
					 ((_11 * kMatrix._13) + (_12 * kMatrix._23) + (_13 * kMatrix._33) + (_14 * kMatrix._43)),
					 ((_11 * kMatrix._14) + (_12 * kMatrix._24) + (_13 * kMatrix._34) + (_14 * kMatrix._44)),

					 ((_21 * kMatrix._11) + (_22 * kMatrix._21) + (_23 * kMatrix._31) + (_24 * kMatrix._41)),
					 ((_21 * kMatrix._12) + (_22 * kMatrix._22) + (_23 * kMatrix._32) + (_24 * kMatrix._42)),
					 ((_21 * kMatrix._13) + (_22 * kMatrix._23) + (_23 * kMatrix._33) + (_24 * kMatrix._43)),
					 ((_21 * kMatrix._14) + (_22 * kMatrix._24) + (_23 * kMatrix._34) + (_24 * kMatrix._44)),

					 ((_31 * kMatrix._11) + (_32 * kMatrix._21) + (_33 * kMatrix._31) + (_34 * kMatrix._41)),
					 ((_31 * kMatrix._12) + (_32 * kMatrix._22) + (_33 * kMatrix._32) + (_34 * kMatrix._42)),
					 ((_31 * kMatrix._13) + (_32 * kMatrix._23) + (_33 * kMatrix._33) + (_34 * kMatrix._43)),
					 ((_31 * kMatrix._14) + (_32 * kMatrix._24) + (_33 * kMatrix._34) + (_34 * kMatrix._44)),

					 ((_41 * kMatrix._11) + (_42 * kMatrix._21) + (_43 * kMatrix._31) + (_44 * kMatrix._41)),
					 ((_41 * kMatrix._12) + (_42 * kMatrix._22) + (_43 * kMatrix._32) + (_44 * kMatrix._42)),
					 ((_41 * kMatrix._13) + (_42 * kMatrix._23) + (_43 * kMatrix._33) + (_44 * kMatrix._43)),
					 ((_41 * kMatrix._14) + (_42 * kMatrix._24) + (_43 * kMatrix._34) + (_44 * kMatrix._44)));
}

void GEKMATRIX::operator *= (const GEKMATRIX &kMatrix)
{
	*this =  GEKMATRIX(((_11 * kMatrix._11) + (_12 * kMatrix._21) + (_13 * kMatrix._31) + (_14 * kMatrix._41)),
					   ((_11 * kMatrix._12) + (_12 * kMatrix._22) + (_13 * kMatrix._32) + (_14 * kMatrix._42)),
					   ((_11 * kMatrix._13) + (_12 * kMatrix._23) + (_13 * kMatrix._33) + (_14 * kMatrix._43)),
					   ((_11 * kMatrix._14) + (_12 * kMatrix._24) + (_13 * kMatrix._34) + (_14 * kMatrix._44)),

					   ((_21 * kMatrix._11) + (_22 * kMatrix._21) + (_23 * kMatrix._31) + (_24 * kMatrix._41)),
					   ((_21 * kMatrix._12) + (_22 * kMatrix._22) + (_23 * kMatrix._32) + (_24 * kMatrix._42)),
					   ((_21 * kMatrix._13) + (_22 * kMatrix._23) + (_23 * kMatrix._33) + (_24 * kMatrix._43)),
					   ((_21 * kMatrix._14) + (_22 * kMatrix._24) + (_23 * kMatrix._34) + (_24 * kMatrix._44)),

					   ((_31 * kMatrix._11) + (_32 * kMatrix._21) + (_33 * kMatrix._31) + (_34 * kMatrix._41)),
					   ((_31 * kMatrix._12) + (_32 * kMatrix._22) + (_33 * kMatrix._32) + (_34 * kMatrix._42)),
					   ((_31 * kMatrix._13) + (_32 * kMatrix._23) + (_33 * kMatrix._33) + (_34 * kMatrix._43)),
					   ((_31 * kMatrix._14) + (_32 * kMatrix._24) + (_33 * kMatrix._34) + (_34 * kMatrix._44)),

					   ((_41 * kMatrix._11) + (_42 * kMatrix._21) + (_43 * kMatrix._31) + (_44 * kMatrix._41)),
					   ((_41 * kMatrix._12) + (_42 * kMatrix._22) + (_43 * kMatrix._32) + (_44 * kMatrix._42)),
					   ((_41 * kMatrix._13) + (_42 * kMatrix._23) + (_43 * kMatrix._33) + (_44 * kMatrix._43)),
					   ((_41 * kMatrix._14) + (_42 * kMatrix._24) + (_43 * kMatrix._34) + (_44 * kMatrix._44)));
}

void GEKMATRIX::Rotate(const GEKMATRIX &kMatrix)
{
	GEKMATRIX kBase(*this);
	_11 = ((kBase._11 * kMatrix._11) + (kBase._12 * kMatrix._21) + (kBase._13 * kMatrix._31));
	_12 = ((kBase._11 * kMatrix._12) + (kBase._12 * kMatrix._22) + (kBase._13 * kMatrix._32));
	_13 = ((kBase._11 * kMatrix._13) + (kBase._12 * kMatrix._23) + (kBase._13 * kMatrix._33));

	_21 = ((kBase._21 * kMatrix._11) + (kBase._22 * kMatrix._21) + (kBase._23 * kMatrix._31));
	_22 = ((kBase._21 * kMatrix._12) + (kBase._22 * kMatrix._22) + (kBase._23 * kMatrix._32));
	_23 = ((kBase._21 * kMatrix._13) + (kBase._22 * kMatrix._23) + (kBase._23 * kMatrix._33));

	_31 = ((kBase._31 * kMatrix._11) + (kBase._32 * kMatrix._21) + (kBase._33 * kMatrix._31));
	_32 = ((kBase._31 * kMatrix._12) + (kBase._32 * kMatrix._22) + (kBase._33 * kMatrix._32));
	_33 = ((kBase._31 * kMatrix._13) + (kBase._32 * kMatrix._23) + (kBase._33 * kMatrix._33));
}

void GEKMATRIX::SetEuler(const GEKVECTOR3 &kEuler)
{
	SetIdentity();
	float fCOSx = COS(kEuler.x);
	float fSINx = SIN(kEuler.x);
	float fCOSy = COS(kEuler.y);
	float fSINy = SIN(kEuler.y);
	float fCOSz = COS(kEuler.z);
	float fSINz = SIN(kEuler.z);

	_11 = (fCOSy * fCOSz);
	_12 = (fCOSy * fSINz);
	_13 = (-fSINy);

	float fSINxSINy = (fSINx * fSINy);
	float fCOSxSINy = (fCOSx * fSINy);

	_21 = ((fSINxSINy * fCOSz) - (fCOSx * fSINz));
	_22 = ((fSINxSINy * fSINz) + (fCOSx * fCOSz));
	_23 = (fSINx * fCOSy);

	_31 = ((fCOSxSINy * fCOSz) + (fSINx * fSINz));
	_32 = ((fCOSxSINy * fSINz) - (fSINx * fCOSz));
	_33 = (fCOSx * fCOSy);
}

void GEKMATRIX::SetRotation(const GEKVECTOR3 &kAxis, float fAngle)
{
    float fCOS = COS(fAngle);
    float fSIN = SIN(fAngle);
	GEKVECTOR3 kNormal(((GEKVECTOR3 &)kAxis).Normal());

    _11 = ((kNormal.x * kNormal.x) * (1.0f - fCOS) + fCOS);
    _12 = ((kNormal.x * kNormal.y) * (1.0f - fCOS) - (kNormal.z * fSIN));
    _13 = ((kNormal.x * kNormal.z) * (1.0f - fCOS) + (kNormal.y * fSIN));

    _21 = ((kNormal.y * kNormal.x) * (1.0f - fCOS) + (kNormal.z * fSIN));
    _22 = ((kNormal.y * kNormal.y) * (1.0f - fCOS) + fCOS);
    _23 = ((kNormal.y * kNormal.z) * (1.0f - fCOS) - (kNormal.x * fSIN));

    _31 = ((kNormal.z * kNormal.x) * (1.0f - fCOS) - (kNormal.y * fSIN));
    _32 = ((kNormal.z * kNormal.y) * (1.0f - fCOS) + (kNormal.x * fSIN));
    _33 = ((kNormal.z * kNormal.z) * (1.0f - fCOS) + fCOS);
}

void GEKMATRIX::SetQuaternion(const GEKQUATERNION &kQuat)
{
	float f2X = (kQuat.x * 2.0f);
	float f2Y = (kQuat.y * 2.0f);
	float f2Z = (kQuat.z * 2.0f);
	float fWX = (kQuat.w * f2X);
	float fWY = (kQuat.w * f2Y);
	float fWZ = (kQuat.w * f2Z);
	float fXX = (kQuat.x * f2X);
	float fXY = (kQuat.x * f2Y);
	float fXZ = (kQuat.x * f2Z);
	float fYY = (kQuat.y * f2Y);
	float fYZ = (kQuat.y * f2Z);
	float fZZ = (kQuat.z * f2Z);
	
	_11 = (1.0f - fYY - fZZ);
	_12 = (fXY + fWZ);
	_13 = (fXZ - fWY);

	_21 = (fXY - fWZ);
	_22 = (1.0f - fXX - fZZ);
	_23 = (fYZ + fWX);

	_31 = (fXZ + fWY);
	_32 = (fYZ - fWX);
	_33 = (1.0f - fXX - fYY);
}

void GEKMATRIX::Transpose(void)
{
	GEKMATRIX kMatrix(*this);
    _11 = kMatrix._11;    _12 = kMatrix._21;    _13 = kMatrix._31;    _14 = kMatrix._41;
    _21 = kMatrix._12;    _22 = kMatrix._22;    _23 = kMatrix._32;    _24 = kMatrix._42;
    _31 = kMatrix._13;    _32 = kMatrix._23;    _33 = kMatrix._33;    _34 = kMatrix._43;
    _41 = kMatrix._14;    _42 = kMatrix._24;    _43 = kMatrix._34;    _44 = kMatrix._44;
}

GEKMATRIX GEKMATRIX::Inverse(void)
{
/*	if((ABS(_44 - 1.0f) > 0.001f)||
	   (ABS(_14) > 0.001f)||
	   (ABS(_24) > 0.001f)||
	   (ABS(_34) > 0.001f)) return *this;*/

	GEKMATRIX kResult;
	float fDetInverse = (1.0f / Determinant());
    kResult._11 = ( fDetInverse * ((_22 * _33) - (_23 * _32)));
    kResult._12 = (-fDetInverse * ((_12 * _33) - (_13 * _32)));
    kResult._13 = ( fDetInverse * ((_12 * _23) - (_13 * _22)));
    kResult._14 = 0.0f;

    kResult._21 = (-fDetInverse * ((_21 * _33) - (_23 * _31)));
    kResult._22 = ( fDetInverse * ((_11 * _33) - (_13 * _31)));
    kResult._23 = (-fDetInverse * ((_11 * _23) - (_13 * _21)));
    kResult._24 = 0.0f;

    kResult._31 = ( fDetInverse * ((_21 * _32) - (_22 * _31)));
    kResult._32 = (-fDetInverse * ((_11 * _32) - (_12 * _31)));
    kResult._33 = ( fDetInverse * ((_11 * _22) - (_12 * _21)));
    kResult._34 = 0.0f;

    kResult._41 = -((_41 * kResult._11) + (_42 * kResult._21) + (_43 * kResult._31));
    kResult._42 = -((_41 * kResult._12) + (_42 * kResult._22) + (_43 * kResult._32));
    kResult._43 = -((_41 * kResult._13) + (_42 * kResult._23) + (_43 * kResult._33));
    kResult._44 = 1.0f;
	return kResult;
}

void GEKMATRIX::Invert(void)
{
	*this = Inverse();
}

GEKVECTOR3 GEKMATRIX::Rotate(const GEKVECTOR2 &kVector)
{
	return GEKVECTOR3(((kVector.x * _11) + (kVector.y * _21)),
					  ((kVector.x * _12) + (kVector.y * _22)),
					  ((kVector.x * _13) + (kVector.y * _23)));
}

GEKVECTOR3 GEKMATRIX::Rotate(const GEKVECTOR3 &kVector)
{
	return GEKVECTOR3(((kVector.x * _11) + (kVector.y * _21) + (kVector.z * _31)),
					  ((kVector.x * _12) + (kVector.y * _22) + (kVector.z * _32)),
					  ((kVector.x * _13) + (kVector.y * _23) + (kVector.z * _33)));
}

GEKVECTOR3 GEKMATRIX::Rotate(const GEKVECTOR4 &kVector)
{
	return GEKVECTOR3(((kVector.x * _11) + (kVector.y * _21) + (kVector.z * _31)),
					  ((kVector.x * _12) + (kVector.y * _22) + (kVector.z * _32)),
					  ((kVector.x * _13) + (kVector.y * _23) + (kVector.z * _33)));
}

GEKVECTOR3 GEKMATRIX::Rotate(const GEKVECTOR2 &kVector) const
{
	return GEKVECTOR3(((kVector.x * _11) + (kVector.y * _21)),
					  ((kVector.x * _12) + (kVector.y * _22)),
					  ((kVector.x * _13) + (kVector.y * _23)));
}

GEKVECTOR3 GEKMATRIX::Rotate(const GEKVECTOR3 &kVector) const
{
	return GEKVECTOR3(((kVector.x * _11) + (kVector.y * _21) + (kVector.z * _31)),
					  ((kVector.x * _12) + (kVector.y * _22) + (kVector.z * _32)),
					  ((kVector.x * _13) + (kVector.y * _23) + (kVector.z * _33)));
}

GEKVECTOR3 GEKMATRIX::Rotate(const GEKVECTOR4 &kVector) const
{
	return GEKVECTOR3(((kVector.x * _11) + (kVector.y * _21) + (kVector.z * _31)),
					  ((kVector.x * _12) + (kVector.y * _22) + (kVector.z * _32)),
					  ((kVector.x * _13) + (kVector.y * _23) + (kVector.z * _33)));
}

GEKVECTOR4 GEKMATRIX::Transform(const GEKVECTOR2 &kVector)
{
	return GEKVECTOR4(((kVector.x * _11) + (kVector.y * _21) + _41),
					  ((kVector.x * _12) + (kVector.y * _22) + _42),
					  ((kVector.x * _13) + (kVector.y * _23) + _43),
					  ((kVector.x * _14) + (kVector.y * _24) + _44));
}

GEKVECTOR4 GEKMATRIX::Transform(const GEKVECTOR3 &kVector)
{
	return GEKVECTOR4(((kVector.x * _11) + (kVector.y * _21) + (kVector.z * _31) + _41),
					  ((kVector.x * _12) + (kVector.y * _22) + (kVector.z * _32) + _42),
					  ((kVector.x * _13) + (kVector.y * _23) + (kVector.z * _33) + _43),
					  ((kVector.x * _14) + (kVector.y * _24) + (kVector.z * _34) + _44));
}

GEKVECTOR4 GEKMATRIX::Transform(const GEKVECTOR4 &kVector)
{
	return GEKVECTOR4(((kVector.x * _11) + (kVector.y * _21) + (kVector.z * _31) + (kVector.w * _41)),
					  ((kVector.x * _12) + (kVector.y * _22) + (kVector.z * _32) + (kVector.w * _42)),
					  ((kVector.x * _13) + (kVector.y * _23) + (kVector.z * _33) + (kVector.w * _43)),
					  ((kVector.x * _14) + (kVector.y * _24) + (kVector.z * _34) + (kVector.w * _44)));
}

GEKVECTOR4 GEKMATRIX::Transform(const GEKVECTOR2 &kVector) const
{
	return GEKVECTOR4(((kVector.x * _11) + (kVector.y * _21) + _41),
					  ((kVector.x * _12) + (kVector.y * _22) + _42),
					  ((kVector.x * _13) + (kVector.y * _23) + _43),
					  ((kVector.x * _14) + (kVector.y * _24) + _44));
}

GEKVECTOR4 GEKMATRIX::Transform(const GEKVECTOR3 &kVector) const
{
	return GEKVECTOR4(((kVector.x * _11) + (kVector.y * _21) + (kVector.z * _31) + _41),
					  ((kVector.x * _12) + (kVector.y * _22) + (kVector.z * _32) + _42),
					  ((kVector.x * _13) + (kVector.y * _23) + (kVector.z * _33) + _43),
					  ((kVector.x * _14) + (kVector.y * _24) + (kVector.z * _34) + _44));
}

GEKVECTOR4 GEKMATRIX::Transform(const GEKVECTOR4 &kVector) const
{
	return GEKVECTOR4(((kVector.x * _11) + (kVector.y * _21) + (kVector.z * _31) + (kVector.w * _41)),
					  ((kVector.x * _12) + (kVector.y * _22) + (kVector.z * _32) + (kVector.w * _42)),
					  ((kVector.x * _13) + (kVector.y * _23) + (kVector.z * _33) + (kVector.w * _43)),
					  ((kVector.x * _14) + (kVector.y * _24) + (kVector.z * _34) + (kVector.w * _44)));
}

GEKMATRIX GEKMatrixOrtho(float fXMin, float fYMin, float fXMax, float fYMax, float fNear, float fFar)
{
	float fX = (2.0f / (fXMax - fXMin));
	float fY = (2.0f / (fYMax - fYMin));
	float fZ = (-2.0f / (fFar - fNear));
	float fTX = -((fXMax + fXMin) / (fXMax - fXMin));
	float fTY = -((fYMax + fYMin) / (fYMax - fYMin));
	float fTZ = -((fFar + fNear) / (fFar - fNear));

	return GEKMATRIX(  fX, 0.0f, 0.0f, 0.0f,
					 0.0f,  -fY, 0.0f, 0.0f,
					 0.0f, 0.0f,  -fZ, 0.0f,
					  fTX, -fTY, -fTZ, 1.0f);
}

GEKMATRIX GEKMatrixPerspective(float fFOV, float fAspect, float fNear, float fFar)
{
	float fYMax = (fNear * (SIN(fFOV / 2.0f) / COS(fFOV / 2.0f)));
	float fXMax = (fYMax * fAspect);
	float fYMin = -fYMax;
	float fXMin = -fXMax;

	float fX = ((2.0f * fNear) / (fXMax - fXMin));
	float fY = ((2.0f * fNear) / (fYMax - fYMin));
	float fA = ((fXMax + fXMin) / (fXMax - fXMin));
	float fB = ((fYMax + fYMin) / (fYMax - fYMin));
	float fC = -((fFar + fNear) / (fFar - fNear));
	float fD = -((2.0f * fFar * fNear) / (fFar - fNear));

	return GEKMATRIX(  fX, 0.0f,   fA, 0.0f,
					 0.0f,   fY,   fB, 0.0f,
					 0.0f, 0.0f,  -fC,  -fD,
					 0.0f, 0.0f,-1.0f, 0.0f);
}

GEKMATRIX GEKMatrixMirror(const GEKPLANE &kPlane)
{
	float fX = kPlane.n.x;
	float fY = kPlane.n.y;
	float fZ = kPlane.n.z;
	float fD = kPlane.d;

	return GEKMATRIX((0.0f - (2.0f * fX * fX)), (0.0f - (2.0f * fX * fY)), (0.0f - (2.0f * fX * fZ)), 0.0f,
					 (0.0f - (2.0f * fY * fX)), (0.0f - (2.0f * fY * fY)), (0.0f - (2.0f * fY * fZ)), 0.0f,
					 (0.0f - (2.0f * fZ * fX)), (0.0f - (2.0f * fZ * fY)), (0.0f - (2.0f * fZ * fZ)), 0.0f,
					 (0.0f - (2.0f * fD * fX)), (0.0f - (2.0f * fD * fY)), (0.0f - (2.0f * fD * fZ)), 1.0f);
}

GEKMATRIX GEKMatrixTarget(const GEKVECTOR3 &kOrigin, const GEKVECTOR3 &kTarget)
{
	GEKVECTOR3 kZAxis(kTarget - kOrigin);
	kZAxis.Normalize();

	GEKVECTOR3 m_kYAxis(0.0f, 1.0f, 0.0f);
	GEKVECTOR3 kXAxis(m_kYAxis.Cross(kZAxis));
	kXAxis.Normalize();

	GEKVECTOR3 kYAxis(kZAxis.Cross(kXAxis));

	return GEKMATRIX(kXAxis.x, kXAxis.y, kXAxis.z, -kXAxis.Dot(kOrigin),
					 kYAxis.x, kYAxis.y, kYAxis.z, -kYAxis.Dot(kOrigin),
					 kZAxis.x, kZAxis.y, kZAxis.z, -kZAxis.Dot(kOrigin),
					 0.0f, 0.0f, 0.0f, 1.0f);
}